import QtQuick 2.12
import QtQuick.Window 2.12
import QtQuick.Controls 2.12
import QtQuick.Layouts 1.12

Window {
    id: mainWindow
    width: Screen.width
    height: Screen.height
    visible: true
    visibility: Window.Maximized
    title: qsTr("Motus")

    // Propriétés liées au jeu
    property int nb_lettres: 5
    property int nb_essais: 6
    property var caseArray: []   // Rempli lors de la création de la grille
    property var case_focus: null
    property int current_essai: 0
    property int indice_case: 0
    property bool statsUpdated: false

    property string mot: ""
    property var mot_split: []

    property var keysArray: []

    // Propriétés pour l'affichage des statistiques
    property bool statsOverlayVisible: false
    // Nombre de lettres pour afficher les stats locales dans l'overlay (entre 4 et 12)
    property int selectedLetterForStats: 5
    // Choix de la langue pour les statistiques ("Combined", "French" ou "English")
    property string language: "Combined"

    // Fonction utilitaire pour changer la couleur d'une touche
    function changeKeyColor(letter, newColor) {
        for (var i = 0; i < keysArray.length; i++) {
            if (keysArray[i].keyLetter === letter) {
                keysArray[i].keyColor = newColor;
            }
        }
    }

    // Fonction pour rafraîchir l'affichage de l'overlay de statistiques
    function refreshStatsDisplay() {
        var globalStats = motusGame.getGlobalStats("Combined");
        partiesJouees.text = "Parties jouées : " + globalStats.totalGames;
        var globalWinPercentage = 0;
        if (globalStats.totalGames > 0) {
            globalWinPercentage = Math.round((globalStats.totalWins / globalStats.totalGames) * 100);
        }
        victoires.text = "Taux de réussite : " + globalWinPercentage + "%";
        winStreakActuel.text = "Win Streak Actuel : " + globalStats.currentWinStreak;
        meilleurStreak.text = "Meilleur Streak : " + globalStats.bestWinStreak;

        var localStats = motusGame.getLocalStats(selectedLetterForStats, language);
        tempsMoyen.text = "Temps moyen : " + localStats.averageTime + " sec";
        meilleurTemps.text = "Meilleur temps : " + localStats.bestTime + " sec";
        partiesJoueesLocal.text = "Parties jouées : " + localStats.gamesPlayed;
        var localWinPercentage = 0;
        if (localStats.gamesPlayed > 0) {
            localWinPercentage = Math.round((localStats.gamesWon / localStats.gamesPlayed) * 100);
        }
        partiesGagnees.text = "Taux de réussite : " + localWinPercentage + "%";
        winStreakActuelLocal.text = "Win Streak Actuel : " + localStats.currentWinStreak;
        meilleurStreakLocal.text = "Meilleur Streak : " + localStats.bestWinStreak;
        // Mise à jour de l'histogramme via la nouvelle propriété "histData"
        histogramRow.histData = localStats.attemptsHistogram;
    }

    // Fonction de mise à jour des statistiques à la fin d'une partie
    function updateGameStats() {
        if (statsUpdated)
            return;

        statsUpdated = true;  // On indique qu'on a déjà mis à jour les stats.
        var lang = (motusGame.dictionnaryChoosed.indexOf("francais") >= 0) ? "French" : "English";
        var letterCount = nb_lettres;
        var won = motusGame.win;
        var attemptsUsed = won ? current_essai + 1 : 0;
        var gameTime = won ? (motusGame.duree_timer - motusGame.remainingTime) : 0;
        motusGame.updateStats(letterCount, lang, won, attemptsUsed, gameTime);
        motusGame.saveStats();  // Sauvegarde immédiate
        refreshStatsDisplay();
    }

    // Overlay des statistiques
    Rectangle {
        id: statsOverlay
        anchors.fill: parent
        color: "#323232AA"
        visible: statsOverlayVisible
        z: 100
        onVisibleChanged: {
            if (!visible) {
                mainRect.forceActiveFocus();
            } else {
                refreshStatsDisplay();
            }
        }

        // Dialogue centré
        Rectangle {
            id: statsDialog
            width: parent.width * 0.6
            height: parent.height * 0.7
            color: "#323232"
            opacity: 0.95
            radius: 10
            anchors.centerIn: parent

            ColumnLayout {
                anchors.fill: parent
                anchors.margins: 20
                spacing: 20

                // En-tête
                RowLayout {
                    width: parent.width
                    Layout.alignment: Qt.AlignTop
                    Text {
                        text: "Statistiques"
                        font.pixelSize: 32
                        color: "white"
                        font.bold: true
                        Layout.fillWidth: true
                    }
                    Button {
                        text: "X"
                        font.pixelSize: 24
                        background: Rectangle {
                            color: "transparent"
                            border.color: "white"
                            radius: 5
                        }
                        onClicked: {
                            statsOverlayVisible = false;
                            mainRect.forceActiveFocus();
                        }
                    }
                }

                // Statistiques Globales
                GroupBox {
                    title: "Statistiques Globales"
                    Layout.fillWidth: true
                    ColumnLayout {
                        spacing: 5
                        Text { id: partiesJouees; color: "white"; font.pixelSize: 20 }
                        Text { id: victoires; color: "white"; font.pixelSize: 20 }
                        Text { id: winStreakActuel; color: "white"; font.pixelSize: 20 }
                        Text { id: meilleurStreak; color: "white"; font.pixelSize: 20 }
                    }
                }

                // Sélecteur et Statistiques Locales
                RowLayout {
                    spacing: 10
                    Layout.fillWidth: true
                    Text {
                        text: "Stats locales pour"
                        font.pixelSize: 20
                        color: "white"
                    }
                    ComboBox {
                        id: letterSelector
                        // Modèle de 4 à 12 lettres
                        model: [4, 5, 6, 7, 8, 9, 10, 11, 12]
                        currentIndex: 1  // Par défaut, 5 lettres
                        font.pixelSize: 20
                        onCurrentIndexChanged: {
                            selectedLetterForStats = model[currentIndex];
                            refreshStatsDisplay();
                        }
                    }
                    Text {
                        text: "lettres"
                        font.pixelSize: 20
                        color: "white"
                    }
                }

                // Affichage des statistiques locales
                GroupBox {
                    title: "Statistiques Locales (" + selectedLetterForStats + " lettres)"
                    Layout.fillWidth: true
                    ColumnLayout {
                        spacing: 5
                        Text { id: tempsMoyen; color: "white"; font.pixelSize: 20 }
                        Text { id: meilleurTemps; color: "white"; font.pixelSize: 20 }
                        Text { id: partiesJoueesLocal; color: "white"; font.pixelSize: 20 }
                        Text { id: partiesGagnees; color: "white"; font.pixelSize: 20 }
                        Text { id: winStreakActuelLocal; color: "white"; font.pixelSize: 20 }
                        Text { id: meilleurStreakLocal; color: "white"; font.pixelSize: 20 }
                    }
                }

                // Histogramme des victoires
                GroupBox {
                    title: "Histogramme des victoires"
                    Layout.fillWidth: true
                    contentItem: RowLayout {
                        id: histogramRow
                        spacing: 10
                        // Propriété renommée pour stocker les données de l'histogramme
                        property var histData: []
                        // Calcul dynamique de la valeur maximum dans histData
                        property int maxValue: {
                            var m = 0;
                            for (var i = 0; i < histData.length; i++) {
                                if (histData[i] > m)
                                    m = histData[i];
                            }
                            return m;
                        }
                        Repeater {
                            model: histogramRow.histData.length
                            delegate: ColumnLayout {
                                spacing: 2
                                Rectangle {
                                    width: 20
                                    height: histogramRow.maxValue > 0 ? Math.max(10, Math.round((histogramRow.histData[index] / histogramRow.maxValue) * 200)) : 10
                                    color: "#4a4a4a"
                                    border.width: 1
                                    border.color: "white"
                                }
                                Text {
                                    text: "Essai " + (index + 1) + "\n(" + histogramRow.histData[index] + ")"
                                    font.pixelSize: 14
                                    color: "white"
                                    horizontalAlignment: Text.AlignHCenter
                                    wrapMode: Text.WordWrap
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    // Zone de jeu principale
    Rectangle {
        id: mainRect
        anchors.fill: parent
        color: "#323232"
        focus: true

        Keys.onPressed: function(event) {
            // Si le jeu est en cours
            if (mot !== "" && !motusGame.loosetry && !motusGame.win && !motusGame.loosetime && !motusGame.looseabandon) {
                if (/^[a-zA-Z]$/.test(event.text)) {
                    if (indice_case < nb_lettres) {
                        var cellIndex = current_essai * nb_lettres + indice_case;
                        caseArray[cellIndex]._textText = event.text.toUpperCase();
                        indice_case++;
                        event.accepted = true;
                    }
                } else if (event.key === Qt.Key_Backspace) {
                    if (indice_case > 0) {
                        indice_case--;
                        var cellIndex = current_essai * nb_lettres + indice_case;
                        caseArray[cellIndex]._textText = "";
                        event.accepted = true;
                    }
                } else if (event.key === Qt.Key_Return) {
                    if (indice_case === nb_lettres) {
                        var candidate = [];
                        for (var i = 0; i < nb_lettres; i++){
                            candidate[i] = caseArray[current_essai * nb_lettres + i]._textText.toUpperCase();
                        }
                        var mot_entre = candidate.join("");
                        if (motusGame.existWord(mot_entre)) {
                            var target = mot_split.map(function(letter) { return letter.toUpperCase(); });
                            var targetCounts = {};
                            for (var i = 0; i < nb_lettres; i++) {
                                var letter = target[i];
                                targetCounts[letter] = (targetCounts[letter] || 0) + 1;
                            }
                            var resultColors = new Array(nb_lettres);
                            for (var i = 0; i < nb_lettres; i++) {
                                if (candidate[i] === target[i]) {
                                    resultColors[i] = "#42cc3d";
                                    targetCounts[candidate[i]]--;
                                } else {
                                    resultColors[i] = "";
                                }
                            }
                            if (mot_entre.toUpperCase() === mot.toUpperCase()) {
                                motusGame.setWinbool(true);
                                updateGameStats();
                            }
                            for (var i = 0; i < nb_lettres; i++) {
                                if (resultColors[i] === "") {
                                    var letter = candidate[i];
                                    if (targetCounts[letter] && targetCounts[letter] > 0) {
                                        resultColors[i] = "#f0d437";
                                        targetCounts[letter]--;
                                    }
                                }
                            }
                            for (var i = 0; i < nb_lettres; i++) {
                                var case_verif = caseArray[current_essai * nb_lettres + i];
                                if (resultColors[i] !== "") {
                                    case_verif.rectangleColor = resultColors[i];
                                    case_verif.rectangleBorderColor = resultColors[i];
                                    changeKeyColor(candidate[i], resultColors[i]);
                                } else {
                                    changeKeyColor(candidate[i], "#323232");
                                }
                            }
                            if (current_essai < nb_essais - 1) {
                                current_essai++;
                                indice_case = 0;
                                case_focus = caseArray[current_essai * nb_lettres];
                            } else {
                                if (mot_entre.toUpperCase() !== mot.toUpperCase()) {
                                    motusGame.setLoosetrybool(true);
                                    updateGameStats();
                                }
                            }
                        } else {
                            for (var i = 0; i < nb_lettres; i++) {
                                var case_verif = caseArray[current_essai * nb_lettres + i];
                                case_verif.rectangleColor = "#cf1b2a";
                                case_verif.rectangleBorderColor = "#cf1b2a";
                            }
                            revertTimer.start();
                        }
                        event.accepted = true;
                    }
                }
            }
        }

        Timer {
            id: revertTimer
            interval: 1000
            repeat: false
            onTriggered: {
                for (var i = 0; i < nb_lettres; i++) {
                    var case_verif = caseArray[current_essai * nb_lettres + i];
                    case_verif.rectangleColor = "#323232";
                    case_verif.rectangleBorderColor = "#ffffff";
                }
            }
        }

        Row {
            id: main_row
            anchors.fill: parent
            spacing: 150

            Column {
                id: leftColumn
                width: 310
                height: 700
                spacing: 40
                anchors.verticalCenter: parent.verticalCenter
                anchors.right: rectangle_center.left
                anchors.rightMargin: 50

                Choosebutton {
                    id: choosebutton
                    height: 90
                    anchors.horizontalCenter: parent.horizontalCenter
                    _textText: "Generate a word"
                    MouseArea {
                        anchors.fill: parent
                        cursorShape: Qt.PointingHandCursor
                        enabled: motusGame && (motusGame.win || motusGame.loosetry || motusGame.looseabandon || motusGame.loosetime || !motusGame.debut)
                        onClicked: {
                            statsUpdated = false;
                            current_essai = 0;
                            indice_case = 0;
                            // Réinitialisation de la grille et des touches
                            for (var i = 0; i < caseArray.length; i++) {
                                caseArray[i]._textText = "";
                                caseArray[i].rectangleColor = "#323232";
                                caseArray[i].rectangleBorderColor = "#ffffff";
                            }
                            for (var j = 0; j < keysArray.length; j++) {
                                keysArray[j].keyColor = "#808080";
                            }
                            motusGame.debut = true;
                            // Génération d'un mot avec nb_lettres (entre 4 et 12)
                            mot = motusGame.getRandomWord(nb_lettres);
                            motusGame.setLoosetrybool(false);
                            motusGame.setLoosetimebool(false);
                            motusGame.setLooseabandonbool(false);
                            motusGame.setWinbool(false);

                            // Système d'adaptation du chrono :
                            // Ici, on définit par exemple 10 secondes de temps par lettre.
                            var baseTimePerLetter = 10;
                            motusGame.duree_timer = nb_lettres * baseTimePerLetter;

                            motusGame.startTimer();
                            mot_split = mot.split("");
                        }
                    }
                }

                Rectangle {
                    id: big_rectangle_langue
                    width: 200
                    height: 240
                    color: "#323232"
                    radius: 10
                    border.color: "#ffffff"
                    border.width: 5
                    anchors.horizontalCenter: parent.horizontalCenter

                    Column {
                        id: column
                        anchors.fill: parent
                        anchors.margins: 10
                        spacing: 15

                        Text {
                            id: text_langue
                            y: 100
                            text: "Language"
                            font.pixelSize: 25
                            color: "white"
                            horizontalAlignment: Text.AlignHCenter
                            anchors.horizontalCenter: parent.horizontalCenter
                        }

                        LangageButton {
                            id: langageButton
                            width: 180
                            height: 70
                            anchors.horizontalCenter: parent.horizontalCenter
                            _textText: "Français"
                            rectangleColor: "#7a7a7a"
                            MouseArea {
                                anchors.fill: parent
                                enabled: motusGame && (motusGame.win || motusGame.loosetry || motusGame.looseabandon || motusGame.loosetime || !motusGame.debut)
                                cursorShape: Qt.PointingHandCursor
                                onClicked: {
                                    motusGame.dictionnaryChoosed = "Motus\\mots_francais_bis.txt";
                                    langageButton2.rectangleColor = "#7a7a7a";
                                    langageButton.rectangleColor = "green";
                                    choosebutton._textText = "Générer un mot";
                                    text_langue.text = "Langage";
                                    text_choix_nb_lettres.text = "Nombre de lettres";
                                    text_stats.text = "Statistiques";
                                    text_abandon.text = "Abandonner";
                                    numberLetterBtn.firstItemText = "5";
                                }
                            }
                        }

                        LangageButton {
                            id: langageButton2
                            anchors.horizontalCenter: parent.horizontalCenter
                            _textText: "English"
                            rectangleColor: "green"
                            MouseArea {
                                enabled: motusGame && (motusGame.win || motusGame.loosetry || motusGame.looseabandon || motusGame.loosetime || !motusGame.debut)
                                anchors.fill: parent
                                cursorShape: Qt.PointingHandCursor
                                onClicked: {
                                    motusGame.dictionnaryChoosed = "Motus\\words_alpha.txt";
                                    langageButton2.rectangleColor = "green";
                                    langageButton.rectangleColor = "#7a7a7a";
                                    choosebutton._textText = "Generate a word";
                                    text_langue.text = "Language";
                                    text_choix_nb_lettres.text = "Number of letters";
                                    text_stats.text = "Statistics";
                                    text_abandon.text = "Abandon";
                                    numberLetterBtn.firstItemText = "5";
                                }
                            }
                        }
                    }
                }

                Rectangle {
                    id: big_rectangle_nb_lettres
                    width: 200
                    height: 130
                    color: "#323232"
                    radius: 10
                    border.color: "#ffffff"
                    border.width: 5
                    anchors.horizontalCenter: parent.horizontalCenter

                    Column {
                        id: colonne_nb_lettres
                        anchors.verticalCenter: parent.verticalCenter
                        anchors.horizontalCenter: parent.horizontalCenter
                        spacing: 20

                        Text {
                            id: text_choix_nb_lettres
                            text: "Number of letters"
                            font.pixelSize: 22
                            anchors.horizontalCenter: parent.horizontalCenter
                            color: "white"
                        }

                        ComboBox {
                            id: comboBox
                            width: 150
                            height: 50
                            anchors.horizontalCenter: parent.horizontalCenter
                            currentIndex: 1
                            // Modèle de 4 à 12 (pas d'aléatoire)
                            model: [4, 5, 6, 7, 8, 9, 10, 11, 12]
                            flat: true
                            enabled: motusGame && (motusGame.win || motusGame.loosetry || motusGame.looseabandon || motusGame.loosetime || !motusGame.debut)

                            contentItem: Text {
                                width: 200
                                text: comboBox.displayText
                                font.pixelSize: 18
                                font.bold: true
                                color: "white"
                                verticalAlignment: Text.AlignVCenter
                                horizontalAlignment: Text.AlignLeft
                                elide: Text.ElideRight
                                anchors.verticalCenter: parent.verticalCenter
                                anchors.left: parent.left
                                anchors.leftMargin: 20
                            }

                            background: Rectangle {
                                radius: 10
                                color: "#7a7a7a"
                                border.color: "white"
                                border.width: 5
                            }

                            indicator: Rectangle {
                                width: 20
                                height: 20
                                anchors.verticalCenter: parent.verticalCenter
                                anchors.right: parent.right
                                anchors.rightMargin: 10
                                color: "transparent"
                                Canvas {
                                    anchors.fill: parent
                                    onPaint: {
                                        var ctx = getContext("2d");
                                        ctx.clearRect(0, 0, width, height);
                                        ctx.beginPath();
                                        ctx.moveTo(4, 6);
                                        ctx.lineTo(width - 4, 6);
                                        ctx.lineTo(width / 2, height - 4);
                                        ctx.closePath();
                                        ctx.fillStyle = "white";
                                        ctx.fill();
                                    }
                                }
                            }

                            delegate: ItemDelegate {
                                width: comboBox.width
                                contentItem: Text {
                                    text: modelData
                                    font.pixelSize: 16
                                    color: "black"
                                }
                            }

                            onCurrentIndexChanged: {
                                nb_lettres = parseInt(model[currentIndex]);
                                // Mettre à jour nb_essais en fonction de nb_lettres
                                if (nb_lettres == 4)
                                    nb_essais = 5;
                                else if (nb_lettres == 5 || nb_lettres == 6)
                                    nb_essais = 6;
                                else if (nb_lettres == 7 || nb_lettres == 8)
                                    nb_essais = 7;
                                else
                                    nb_essais = 8;
                                caseRepeater.model = nb_lettres * nb_essais;
                                caseArray = [];
                                for (var i = 0; i < caseRepeater.count; i++) {
                                    caseArray.push(caseRepeater.itemAt(i));
                                }
                                if (caseArray.length > 0) {
                                    case_focus = caseArray[0];
                                }
                                indice_case = 0;
                                current_essai = 0;
                                mainRect.focus = true;
                            }
                        }
                    }
                }

                Rectangle {
                    id: stats_rect
                    width: 200
                    height: 80
                    color: "#323232"
                    radius: 10
                    border.color: "#ffffff"
                    border.width: 5
                    anchors.horizontalCenter: parent.horizontalCenter

                    Text {
                        id: text_abandon
                        color: "#ffffff"
                        text: qsTr("Abandon")
                        anchors.left: parent.left
                        anchors.top: parent.top
                        anchors.bottom: parent.bottom
                        font.pixelSize: 25
                        horizontalAlignment: Text.AlignHCenter
                        verticalAlignment: Text.AlignVCenter
                        anchors.horizontalCenterOffset: 0
                        anchors.horizontalCenter: parent.horizontalCenter
                        font.family: "Tahoma"
                        MouseArea {
                            anchors.fill: parent
                            cursorShape: Qt.PointingHandCursor
                            enabled: motusGame && (!motusGame.win && !motusGame.loosetry && !motusGame.looseabandon && !motusGame.loosetime)
                            onClicked: {
                                updateGameStats();
                                statsOverlayVisible = true;
                                selectedLetterForStats = nb_lettres;
                                motusGame.looseabandon = true;
                            }
                        }
                    }
                }
            }

            Rectangle {
                id: rectangle_center
                width: 500
                height: 700
                color: "#323232"
                anchors.verticalCenter: parent.verticalCenter
                anchors.horizontalCenter: parent.horizontalCenter

                Column {
                    id: center_column
                    width: 600
                    height: 700
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.horizontalCenter: parent.horizontalCenter
                    spacing: 10

                    Rectangle {
                        id: rectangle_titre
                        width: 210
                        height: 100
                        color: "#323232"
                        radius: 20
                        border.color: "#ffffff"
                        border.width: 5
                        anchors.horizontalCenter: parent.horizontalCenter

                        Text {
                            id: _text2
                            text: qsTr("MOTUS")
                            anchors.verticalCenter: parent.verticalCenter
                            font.pixelSize: 50
                            color: "white"
                            font.family: "Tahoma"
                            font.bold: true
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                            anchors.horizontalCenter: parent.horizontalCenter
                        }
                    }

                    // Grille de jeu
                    Rectangle {
                        id: rectangle_grid
                        width: 450
                        height: 420
                        color: "#323232"
                        anchors.horizontalCenter: parent.horizontalCenter

                        Grid {
                            id: grid
                            height: nb_essais * 50 + (nb_essais - 1) * 5
                            width: nb_lettres * 50 + (nb_lettres - 1) * 5
                            anchors.horizontalCenter: parent.horizontalCenter
                            anchors.verticalCenter: parent.verticalCenter
                            spacing: 5
                            columns: nb_lettres
                            Repeater {
                                id: caseRepeater
                                model: nb_lettres * nb_essais
                                delegate: Case {
                                    objectName: "caseDelegate"
                                }
                            }
                        }

                        MouseArea {
                            id: mouseArea
                            anchors.fill: parent
                            onClicked: {
                                rectangle_grid.focus = true;
                            }
                        }
                    }

                    Rectangle {
                        id: rectangle1
                        width: 500
                        height: 200
                        anchors.horizontalCenter: parent.horizontalCenter
                        color: "#323232"
                        radius: 10
                        border.color: "#ffffff"
                        border.width: 5

                        Column {
                            id: column2
                            width: 500
                            anchors.verticalCenter: parent.verticalCenter
                            anchors.top: parent.top
                            anchors.margins: 10
                            anchors.horizontalCenter: parent.horizontalCenter
                            spacing: 6
                            topPadding: rectangle1.height * 0.05
                            bottomPadding: rectangle1.height * 0.05

                            Repeater {
                                anchors.verticalCenter: parent.verticalCenter
                                model: [
                                    ["A", "Z", "E", "R", "T", "Y", "U", "I", "O", "P"],
                                    ["Q", "S", "D", "F", "G", "H", "J", "K", "L"],
                                    ["↑", "W", "X", "C", "V", "B", "N", "M", "<"]
                                ]
                                delegate: Row {
                                    spacing: 5
                                    anchors.horizontalCenter: parent.horizontalCenter

                                    Repeater {
                                        model: modelData
                                        delegate: Rectangle {
                                            id: keyRect
                                            property string keyLetter: modelData
                                            property color keyColor: "#808080"

                                            width: (modelData === "Entrée" || modelData === "<") ? 70 : 40
                                            height: 50
                                            color: keyColor
                                            radius: 5
                                            border.color: keyColor

                                            Text {
                                                anchors.centerIn: parent
                                                text: modelData
                                                font.pixelSize: 25
                                                color: "white"
                                                font.family: "Tahoma"
                                                font.bold: true
                                            }
                                            MouseArea {
                                                anchors.fill: parent
                                                cursorShape: Qt.PointingHandCursor
                                                onClicked: {
                                                    if (mot !== "" && !motusGame.loosetry && !motusGame.looseabandon && !motusGame.win && !motusGame.loosetime) {
                                                        if (modelData === "<") {
                                                            if (indice_case > 0) {
                                                                indice_case--;
                                                                var cellIndex = current_essai * nb_lettres + indice_case;
                                                                caseArray[cellIndex]._textText = "";
                                                                event.accepted = true;
                                                            }
                                                        } else if (modelData === "↑") {
                                                            if (indice_case === nb_lettres) {
                                                                var candidate = [];
                                                                for (var i = 0; i < nb_lettres; i++){
                                                                    candidate[i] = caseArray[current_essai * nb_lettres + i]._textText.toUpperCase();
                                                                }
                                                                var mot_entre = candidate.join("");
                                                                if (motusGame.existWord(mot_entre)) {
                                                                    var target = mot_split.map(function(letter) { return letter.toUpperCase(); });
                                                                    var targetCounts = {};
                                                                    for (var i = 0; i < nb_lettres; i++) {
                                                                        var letter = target[i];
                                                                        targetCounts[letter] = (targetCounts[letter] || 0) + 1;
                                                                    }
                                                                    var resultColors = new Array(nb_lettres);
                                                                    for (var i = 0; i < nb_lettres; i++) {
                                                                        if (candidate[i] === target[i]) {
                                                                            resultColors[i] = "#42cc3d";
                                                                            targetCounts[candidate[i]]--;
                                                                        } else {
                                                                            resultColors[i] = "";
                                                                        }
                                                                    }
                                                                    if (mot_entre.toUpperCase() === mot.toUpperCase()) {
                                                                        motusGame.setWinbool(true);
                                                                        updateGameStats();
                                                                    }
                                                                    for (var i = 0; i < nb_lettres; i++) {
                                                                        if (resultColors[i] === "") {
                                                                            var letter = candidate[i];
                                                                            if (targetCounts[letter] && targetCounts[letter] > 0) {
                                                                                resultColors[i] = "#f0d437";
                                                                                targetCounts[letter]--;
                                                                            }
                                                                        }
                                                                    }
                                                                    for (var i = 0; i < nb_lettres; i++) {
                                                                        var case_verif = caseArray[current_essai * nb_lettres + i];
                                                                        if (resultColors[i] !== "") {
                                                                            case_verif.rectangleColor = resultColors[i];
                                                                            case_verif.rectangleBorderColor = resultColors[i];
                                                                            changeKeyColor(candidate[i], resultColors[i]);
                                                                        }
                                                                    }
                                                                    if (current_essai < nb_essais - 1) {
                                                                        current_essai++;
                                                                        indice_case = 0;
                                                                        case_focus = caseArray[current_essai * nb_lettres];
                                                                    } else {
                                                                        if (mot_entre.toUpperCase() !== mot.toUpperCase()) {
                                                                            motusGame.setLoosetrybool(true);
                                                                            updateGameStats();
                                                                        }
                                                                    }
                                                                } else {
                                                                    for (var i = 0; i < nb_lettres; i++) {
                                                                        var case_verif = caseArray[current_essai * nb_lettres + i];
                                                                        case_verif.rectangleColor = "#cf1b2a";
                                                                        case_verif.rectangleBorderColor = "#cf1b2a";
                                                                    }
                                                                    revertTimer.start();
                                                                }
                                                            }
                                                        } else {
                                                            if (indice_case < nb_lettres) {
                                                                var cellIndex = current_essai * nb_lettres + indice_case;
                                                                caseArray[cellIndex]._textText = modelData.toUpperCase();
                                                                indice_case++;
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                            Component.onCompleted: {
                                                mainWindow.keysArray.push(keyRect);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            Rectangle {
                id: rectangle_right
                width: 250
                height: 850
                color: "#323232"
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: rectangle_center.right
                anchors.leftMargin: 80

                Column {
                    id: column1
                    width: 250
                    height: 680
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.horizontalCenter: parent.horizontalCenter
                    spacing: 30

                    MotusTimer {
                        id: motusTimer
                        width: 100
                        height: 100
                        anchors.horizontalCenter: parent.horizontalCenter
                    }
                    Rectangle {
                        id: rect_stats
                        width: 250
                        height: 100
                        color: "#323232"
                        radius: 10
                        border.color: "#ffffff"
                        border.width: 5
                        anchors.horizontalCenter: parent.horizontalCenter

                        Text {
                            id: text_stats
                            color: "#ffffff"
                            text: qsTr("Statistics")
                            anchors.left: parent.left
                            anchors.top: parent.top
                            anchors.bottom: parent.bottom
                            font.pixelSize: 35
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                            anchors.horizontalCenter: parent.horizontalCenter
                            font.family: "Tahoma"
                            MouseArea {
                                anchors.fill: parent
                                cursorShape: Qt.PointingHandCursor
                                onClicked: {
                                    updateGameStats();
                                    statsOverlayVisible = true;
                                }
                            }
                        }
                    }

                    Text {
                        visible: motusGame && motusGame.loosetime
                        color: "red"
                        text: "Temps écoulé ! Mot : " + mot.toUpperCase()
                        font.pixelSize: 30
                        horizontalAlignment: Text.AlignHCenter
                        verticalAlignment: Text.AlignVCenter
                        font.bold: true
                        anchors.horizontalCenter: parent.horizontalCenter
                    }

                    Text {
                        visible: motusGame && motusGame.loosetry
                        color: "red"
                        text: "Nombre d'essais \n maximum atteint ! \n Mot : " + mot.toUpperCase()
                        font.pixelSize: 30
                        horizontalAlignment: Text.AlignHCenter
                        verticalAlignment: Text.AlignVCenter
                        font.bold: true
                        anchors.horizontalCenter: parent.horizontalCenter
                    }

                    Text {
                        visible: motusGame && motusGame.looseabandon
                        color: "red"
                        text: "Abandon... \n Mot : " + mot.toUpperCase()
                        font.pixelSize: 30
                        horizontalAlignment: Text.AlignHCenter
                        verticalAlignment: Text.AlignVCenter
                        font.bold: true
                        anchors.horizontalCenter: parent.horizontalCenter
                    }

                    Text {
                        text: "Gagné !"
                        visible: motusGame && motusGame.win
                        color: "green"
                        font.pixelSize: 30
                        horizontalAlignment: Text.AlignHCenter
                        verticalAlignment: Text.AlignVCenter
                        font.bold: true
                        anchors.horizontalCenter: parent.horizontalCenter
                    }
                }
            }

        }

        // Zone de grille dynamique

        Component.onCompleted: {
            caseArray = [];
            for (var i = 0; i < caseRepeater.count; i++) {
                caseArray.push(caseRepeater.itemAt(i));
            }
            if (caseArray.length > 0) {
                case_focus = caseArray[0];
            }
        }
    }
}
