#include "motusgame.h"
#include <QFile>
#include <QTextStream>
#include <QRandomGenerator>
#include <QDebug>
#include <QStandardPaths>
#include <QDir>
#include <QCoreApplication>

// Constructeur
MotusGame::MotusGame(QObject *parent) : QObject(parent) {
    loadWords();
    //qDebug() << "🔧 Connexion timer établie";

    // Initialiser les statistiques pour "English", "French" et "Combined"
    QStringList sections = {"English", "French", "Combined"};
    for (const QString &sec : sections) {
        GameStats gs;
        gs.totalGames = 0;
        gs.totalWins = 0;
        gs.currentWinStreak = 0;
        gs.bestWinStreak = 0;
        for (int len = 4; len <= 12; ++len) {
            LocalStats ls;
            ls.averageTime = 0.0;
            ls.bestTime = 0.0;
            int attempts = 0;
            switch(len) {
            case 4: attempts = 5; break;
            case 5: attempts = 6; break;
            case 6: attempts = 6; break;
            case 7: attempts = 7; break;
            case 8: attempts = 7; break;
            default: attempts = 8; break;
            }
            for (int i = 0; i < attempts; ++i)
                ls.attemptsHistogram.append(0);
            ls.gamesPlayed = 0;
            ls.gamesWon = 0;
            ls.currentWinStreak = 0;
            ls.bestWinStreak = 0;
            gs.localStats.insert(len, ls);
        }
        stats.insert(sec, gs);
    }
    // Charger les statistiques sauvegardées
    loadStats();
    emit statsChanged();

    countdownTimer = new QTimer(this);
    countdownTimer->setInterval(1000);

    connect(countdownTimer, &QTimer::timeout, this, [this]() {
        //qDebug() << "🔁 Timeout déclenché";
        if (remainingTime > 0) {
            remainingTime--;
            emit timerUpdated();
            //qDebug() << "⏱️ Tick:" << remainingTime;
            if (remainingTime == 0) {
                countdownTimer->stop();
                //qDebug() << "⏱️ Temps écoulé !";
                setLoosetimebool(true);
                //qDebug() << "💀 Défaite enregistrée via setLoosetimebool(true)";
            }
        }
    });
    //qDebug() << "🔧 Connexion timer établie";
}

void MotusGame::stopTimer() {
    if (countdownTimer && countdownTimer->isActive()) {
        countdownTimer->stop();
        //qDebug() << "Timer arrêté.";
    }
}

void MotusGame::loadWords() {
    wordsByLength.clear();
    QFile file(dictionnaryChoosed);
    if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        QTextStream in(&file);
        while (!in.atEnd()) {
            QString line = in.readLine().trimmed();
            if (!line.isEmpty()) {
                int len = line.length();
                if (len >= 4 && len <= 12) {
                    wordsByLength[len].append(line);
                }
            }
        }
        file.close();
        int totalWords = 0;
        for (int i = 4; i <= 12; ++i) {
            totalWords += wordsByLength[i].size();
        }
        //qDebug() << "✅ Mots chargés :" << totalWords;
    } else {
        //qDebug() << "❌ Erreur : impossible d'ouvrir" << dictionnaryChoosed;
    }
}

bool MotusGame::existWord(const QString &word) {
    int n = word.size();
    bool exists = false;
    const QStringList &list = wordsByLength[n];
    for (int i = 0; i < list.size(); i++) {
        if (list[i].toUpper() == word) {
            exists = true;
            break;
        }
    }
    return exists;
}

QString MotusGame::getRandomWord(int letterCount) {
    // Accepte désormais les mots de 4 à 12 lettres
    if (letterCount < 4 || letterCount > 12) {
        //qDebug() << "❌ Nombre de lettres invalide. Choisissez un nombre entre 4 et 12.";
        return "";
    }
    if (!wordsByLength.contains(letterCount) || wordsByLength[letterCount].isEmpty()) {
        //qDebug() << "❌ Aucun mot disponible pour" << letterCount << "lettres.";
        return "";
    }
    int index = QRandomGenerator::global()->bounded(wordsByLength[letterCount].size());
    motChoisi = wordsByLength[letterCount][index];
    return motChoisi;
}

QString MotusGame::getDictionnaryChoosed() const {
    return dictionnaryChoosed;
}

void MotusGame::setDictionnaryChoosed(const QString &value) {
    if (dictionnaryChoosed != value) {
        dictionnaryChoosed = value;
        emit dictionnaryChoosedChanged();
        loadWords();
        //qDebug() << "📁 Nouveau dictionnaire sélectionné :" << dictionnaryChoosed;
    }
}

void MotusGame::startTimer() {
    //qDebug() << "🔁 startTimer() appelé";
    remainingTime = duree_timer;
    emit timerUpdated();
    countdownTimer->stop();
    countdownTimer->start();
    //qDebug() << "▶️ Timer actif ?" << countdownTimer->isActive();
    //qDebug() << "▶️ Timer lancé";
    //qDebug() << "⏳ remainingTime =" << remainingTime;
}

int MotusGame::getRemainingTime() const {
    return remainingTime;
}

int MotusGame::getLetterNumber() const {
    return letterNumber;
}

void MotusGame::setLetterNumber(int value) {
    if (letterNumber != value) {
        letterNumber = value;
        // Mise à jour du nombre d'essais en fonction du nombre de lettres
        if (value == 4)
            tryTotal = 5;
        else if (value == 5 || value == 6)
            tryTotal = 6;
        else if (value == 7 || value == 8)
            tryTotal = 7;
        else
            tryTotal = 8;
        emit letterNumberChanged();
        emit tryTotalChanged();
        //qDebug() << "Nouvelle valeur de letterNumber :" << letterNumber << ", tryTotal mis à jour à" << tryTotal;
    }
}

bool MotusGame::getLoosetrybool() const {
    return loosetry;
}

bool MotusGame::getLooseabandonbool() const {
    return looseabandon;
}

bool MotusGame::getWinbool() const {
    return win;
}

bool MotusGame::getLoosetimebool() const {
    return loosetime;
}

void MotusGame::setLoosetimebool(bool value) {
    loosetime = value;
    stopTimer();
    emit loosetimeChanged();
}

void MotusGame::setLoosetrybool(bool value) {
    loosetry = value;
    stopTimer();
    emit loosetryChanged();
}

void MotusGame::setLooseabandonbool(bool value) {
    looseabandon = value;
    stopTimer();
    emit looseabandonChanged();
}

void MotusGame::setWinbool(bool value) {
    win = value;
    stopTimer();
    emit winChanged();
}

void MotusGame::loadStats() {
    QFile file("motus_stats.txt");
    if (!file.exists()) {
        //qDebug() << "Le fichier stats n'existe pas, création d'un fichier par défaut.";
        saveStats();
        return;
    }
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        //qDebug() << "Erreur d'ouverture du fichier stats";
        return;
    }
    QTextStream in(&file);
    QStringList sections = {"English", "French", "Combined"};
    for (const QString &sec : sections) {
        GameStats &gs = stats[sec];
        gs.totalGames = in.readLine().toInt();
        gs.totalWins = in.readLine().toInt();
        gs.currentWinStreak = in.readLine().toInt();
        gs.bestWinStreak = in.readLine().toInt();
        for (int len = 4; len <= 12; ++len) {
            LocalStats ls;
            ls.averageTime = in.readLine().toDouble();
            ls.bestTime = in.readLine().toDouble();
            QString histLine = in.readLine();
            QStringList histValues = histLine.split(",", Qt::SkipEmptyParts);
            ls.attemptsHistogram.clear();
            for (const QString &val : histValues) {
                ls.attemptsHistogram.append(val.toInt());
            }
            ls.gamesPlayed = in.readLine().toInt();
            ls.gamesWon = in.readLine().toInt();
            ls.currentWinStreak = in.readLine().toInt();
            ls.bestWinStreak = in.readLine().toInt();
            gs.localStats[len] = ls;
        }
    }
    file.close();
    //qDebug() << "Statistiques chargées";
    emit statsChanged();
}

void MotusGame::saveStats() {
    QFile file("motus_stats.txt");
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        //qDebug() << "Erreur d'ouverture en écriture du fichier stats";
        return;
    }
    QTextStream out(&file);
    QStringList sections = {"English", "French", "Combined"};
    for (const QString &sec : sections) {
        GameStats &gs = stats[sec];
        out << gs.totalGames << "\n";
        out << gs.totalWins << "\n";
        out << gs.currentWinStreak << "\n";
        out << gs.bestWinStreak << "\n";
        for (int len = 4; len <= 12; ++len) {
            LocalStats &ls = gs.localStats[len];
            out << ls.averageTime << "\n";
            out << ls.bestTime << "\n";
            QStringList histValues;
            for (int val : ls.attemptsHistogram) {
                histValues.append(QString::number(val));
            }
            out << histValues.join(",") << "\n";
            out << ls.gamesPlayed << "\n";
            out << ls.gamesWon << "\n";
            out << ls.currentWinStreak << "\n";
            out << ls.bestWinStreak << "\n";
        }
    }
    file.close();
    //qDebug() << "Statistiques sauvegardées";
}

void MotusGame::updateStats(int letterCount, const QString &language, bool win, int attemptsUsed, int gameTime) {
    QString langKey = language;
    QString combinedKey = "Combined";
    QList<QString> keysToUpdate;
    keysToUpdate << langKey << combinedKey;
    for (const QString &key : keysToUpdate) {
        GameStats &gs = stats[key];
        gs.totalGames++;
        gs.localStats[letterCount].gamesPlayed++;
        if (win) {
            gs.totalWins++;
            gs.localStats[letterCount].gamesWon++;
            gs.currentWinStreak++;
            if (gs.currentWinStreak > gs.bestWinStreak)
                gs.bestWinStreak = gs.currentWinStreak;
            LocalStats &ls = gs.localStats[letterCount];
            ls.currentWinStreak++;
            if (ls.currentWinStreak > ls.bestWinStreak)
                ls.bestWinStreak = ls.currentWinStreak;
            if (attemptsUsed >= 1 && attemptsUsed <= ls.attemptsHistogram.size())
                ls.attemptsHistogram[attemptsUsed - 1] += 1;
            if (ls.gamesWon == 1) {
                ls.averageTime = gameTime;
                ls.bestTime = gameTime;
            } else {
                ls.averageTime = ((ls.averageTime * (ls.gamesWon - 1)) + gameTime) / ls.gamesWon;
                if (gameTime < ls.bestTime)
                    ls.bestTime = gameTime;
            }
        } else {
            gs.currentWinStreak = 0;
            gs.localStats[letterCount].currentWinStreak = 0;
        }
    }
    //qDebug() << "Statistiques mises à jour pour" << language << "avec" << letterCount << "lettres, victoire:" << win;
    saveStats();
    emit statsChanged();
}

QVariantMap MotusGame::getGlobalStats(const QString &section) {
    QVariantMap map;
    if (stats.contains(section)) {
        GameStats gs = stats[section];
        map["totalGames"] = gs.totalGames;
        map["totalWins"] = gs.totalWins;
        map["currentWinStreak"] = gs.currentWinStreak;
        map["bestWinStreak"] = gs.bestWinStreak;
    }
    return map;
}

QVariantMap MotusGame::getLocalStats(int letterCount, const QString &section) {
    QVariantMap map;
    if (stats.contains(section)) {
        const GameStats &gs = stats[section];
        if (gs.localStats.contains(letterCount)) {
            const LocalStats &ls = gs.localStats[letterCount];
            map["averageTime"] = ls.averageTime;
            map["bestTime"] = ls.bestTime;
            map["gamesPlayed"] = ls.gamesPlayed;
            map["gamesWon"] = ls.gamesWon;
            map["currentWinStreak"] = ls.currentWinStreak;
            map["bestWinStreak"] = ls.bestWinStreak;
            QVariantList histo;
            for (int val : ls.attemptsHistogram)
                histo.append(val);
            map["attemptsHistogram"] = histo;
        }
    }
    return map;
}
