#ifndef MOTUSGAME_H
#define MOTUSGAME_H

#include <QObject>
#include <QString>
#include <QStringList>
#include <QMap>
#include <QTimer>
#include <QList>
#include <QVariantMap>

// Structure pour stocker les statistiques locales par nombre de lettres
struct LocalStats {
    double averageTime;
    double bestTime;
    QList<int> attemptsHistogram;  // Histogramme des victoires par nombre d'essais
    int gamesPlayed;
    int gamesWon;
    int currentWinStreak;
    int bestWinStreak;
};

// Structure pour stocker les statistiques globales et locales d'une section (ex : English, French, Combined)
struct GameStats {
    int totalGames;
    int totalWins;
    int currentWinStreak;
    int bestWinStreak;
    QMap<int, LocalStats> localStats;  // Clé = nombre de lettres (de 4 à 12)
};

class MotusGame : public QObject {
    Q_OBJECT

    Q_PROPERTY(QString dictionnaryChoosed READ getDictionnaryChoosed WRITE setDictionnaryChoosed NOTIFY dictionnaryChoosedChanged)
    Q_PROPERTY(int remainingTime READ getRemainingTime NOTIFY timerUpdated)
    Q_PROPERTY(int letterNumber READ getLetterNumber WRITE setLetterNumber NOTIFY letterNumberChanged)
    Q_PROPERTY(bool loosetry READ getLoosetrybool WRITE setLoosetrybool NOTIFY loosetryChanged)
    Q_PROPERTY(bool loosetime READ getLoosetimebool WRITE setLoosetimebool NOTIFY loosetimeChanged)
    Q_PROPERTY(bool looseabandon READ getLooseabandonbool WRITE setLooseabandonbool NOTIFY looseabandonChanged)
    Q_PROPERTY(bool win READ getWinbool WRITE setWinbool NOTIFY winChanged)

public:
    explicit MotusGame(QObject *parent = nullptr);
    Q_INVOKABLE QString getRandomWord(int value);

    QString getDictionnaryChoosed() const;
    void setDictionnaryChoosed(const QString &value);

    Q_INVOKABLE void startTimer();
    int getRemainingTime() const;
    Q_INVOKABLE void stopTimer();

    int getLetterNumber() const;
    void setLetterNumber(int value);
    Q_INVOKABLE int getTryTotal() const { return tryTotal; }

    bool getLoosetrybool() const;
    Q_INVOKABLE void setLoosetrybool(bool value);
    bool getLoosetimebool() const;
    Q_INVOKABLE void setLoosetimebool(bool value);
    bool getLooseabandonbool() const;
    Q_INVOKABLE void setLooseabandonbool(bool value);
    bool getWinbool() const;
    Q_INVOKABLE void setWinbool(bool value);

    Q_INVOKABLE bool existWord(const QString &word);

    // Méthodes pour la gestion des statistiques
    Q_INVOKABLE void loadStats();
    Q_INVOKABLE void saveStats();
    Q_INVOKABLE void updateStats(int letterCount, const QString &language, bool win, int attemptsUsed, int gameTime);

    Q_INVOKABLE QVariantMap getGlobalStats(const QString &section);
    Q_INVOKABLE QVariantMap getLocalStats(int letterCount, const QString &section);

private:
    QStringList words;
    QString motChoisi;
    QMap<int, QStringList> wordsByLength;

    int letterNumber = 5;
    bool randomizeletterNumber = true;
    int tryTotal = 6;
    int tryNumber = 0;
    QString dictionnaryChoosed = "Motus\\words_alpha.txt";

    QTimer *countdownTimer;
    int remainingTime = 120;
    bool loosetry = false;
    bool loosetime = false;
    bool looseabandon = false;
    bool win = false;
    bool debut = false;
    int duree_timer = 120;

    void loadWords();

    QMap<QString, GameStats> stats;  // Clés : "English", "French", "Combined"

signals:
    void dictionnaryChoosedChanged();
    void timerUpdated();
    void letterNumberChanged();
    void loosetryChanged();
    void loosetimeChanged();
    void looseabandonChanged();
    void winChanged();
    void statsChanged();
    void tryTotalChanged();
};

#endif // MOTUSGAME_H
